import path from 'node:path';
import { promises as fsp } from 'node:fs';
import type { Plugin } from 'vite';
import { createCert } from '@kevisual/router/sign';
const defaultCacheDir = 'node_modules/.vite';

interface Options {
  certDir: string;
  http: string;
  download: boolean;
}

function viteBasicSslPlugin(options?: Partial<Options>): Plugin {
  return {
    name: 'vite:basic-ssl',
    async configResolved(config) {
      const certificate = await getCertificate(options?.certDir ?? (config.cacheDir ?? defaultCacheDir) + '/basic-ssl', options);
      const https = () => ({ cert: certificate, key: certificate });
      if (config.server.https === undefined || !!config.server.https) {
        config.server.https = Object.assign({}, config.server.https, https());
      }
      if (config.preview.https === undefined || !!config.preview.https) {
        config.preview.https = Object.assign({}, config.preview.https, https());
      }
    },
  };
}
export const fetchText = async (url: string): Promise<string> => {
  try {
    const res = await fetch(url);
    console.log(`Fetching ${url}...`, res);
    if (res.status !== 200) return '';
    return await res.text();
  } catch (error) {
    console.error(`Failed to fetch ${url}:`, error);
    return '';
  }
};
export async function getCertificate(cacheDir: string, options?: Partial<Options>): Promise<string> {
  const link = options?.http || 'https://kevisual.xiongxiao.me/root/ai/code/config/pem/';
  const cachePath = path.join(cacheDir, '_cert.pem');
  const download = options?.download ?? false;
  try {
    if (download === true) {
      throw new Error('need to download');
    }
    const [content] = await Promise.all([fsp.readFile(cachePath, 'utf8')]);

    return content;
  } catch {
    let content = '';
    if (link) {
      const keyLink = link + 'https-private-key.pem?download=1';
      const certLink = link + 'https-cert.pem?download=1';
      const key = await fetchText(keyLink);
      const cert = await fetchText(certLink);
      if (key && cert) {
        content = key + cert;
      } else {
        console.warn(`Failed to fetch certificate from ${link}. Using default certificate.`);
        console.error(`Key: ${key}, ${keyLink}`);
        console.error(`Cert: ${cert}, ${certLink}`);
      }
    }
    if (!content) {
      const cert = createCert();
      content = cert.key + cert.cert;
    }
    fsp
      .mkdir(cacheDir, { recursive: true })
      .then(() => fsp.writeFile(cachePath, content))
      .catch(() => {});
    return content;
  }
}
export const ssl = viteBasicSslPlugin;

export default viteBasicSslPlugin;
